//
// "00-HeikinAshi_v102.mq4" -- Heikin-Ashi
//
//    Ver. 1.00  2008/11/02(Sun)  initial version
//    Ver. 1.01  2009/09/30(Wed)  cleanup for EA
//    Ver. 1.02  2010/05/19(Wed)  smoothed
// 
//
#property copyright  "00mql4@gmail.com"
#property link       "http://00mql4.blogspot.com/"

//---- defines

//---- indicator settings
#property indicator_chart_window

#property indicator_buffers  4

#property indicator_color1  Crimson
#property indicator_color2  DodgerBlue
#property indicator_color3  Crimson
#property indicator_color4  DodgerBlue

#property indicator_width1  1
#property indicator_width2  1
#property indicator_width3  3
#property indicator_width4  3

#property indicator_style1  STYLE_SOLID
#property indicator_style2  STYLE_SOLID
#property indicator_style3  STYLE_SOLID
#property indicator_style4  STYLE_SOLID

//---- indicator parameters
extern int     TimeFrame  = 0;          // time frame
extern int     MaMethod   = MODE_SMMA;  // 0: MODE_SMA, 1: MODE_EMA, 2: MODE_SMMA, 3: MODE_LWMA
extern int     MaPeriod   = 6;          // MA period
extern int     MaMethod2  = MODE_LWMA;  // 0: MODE_SMA, 1: MODE_EMA, 2: MODE_SMMA, 3: MODE_LWMA
extern int     MaPeriod2  = 2;
extern double  yOffset    = 0;          // y offset for display (pips)
extern int     nMaxBars   = 20000;      // maximum number of bars to calculate, 0: no limit

//---- indicator buffers
double BufferLowAve[];    // 0: haLow
double BufferHighAve[];   // 1: haHigh
double BufferOpenAve[];   // 2: haOpen
double BufferCloseAve[];  // 3: haClose
double BufferLowRaw[];    // 4: haLow
double BufferHighRaw[];   // 5: haHigh
double BufferOpenRaw[];   // 6: haOpen
double BufferCloseRaw[];  // 7: haClose

//---- vars
string sIndicatorName;
string sIndSelf = "00-HeikinAshi_v102";

//----------------------------------------------------------------------
string TimeFrameToStr(int timeFrame)
{
    switch (timeFrame) {
    case 1:     return("M1");
    case 5:     return("M5");
    case 15:    return("M15");
    case 30:    return("M30");
    case 60:    return("H1");
    case 240:   return("H4");
    case 1440:  return("D1");
    case 10080: return("W1");
    case 43200: return("MN");
    }
    
    return("??");
}

//----------------------------------------------------------------------
void init()
{
    if (TimeFrame == 0) {
	TimeFrame = Period();
    }
    
    string tf = TimeFrameToStr(TimeFrame);
    sIndicatorName = sIndSelf + ("(" + tf + "," + MaMethod + "," + MaPeriod + ")");
    
    IndicatorShortName(sIndicatorName);
    
    IndicatorBuffers(8);
    
    SetIndexBuffer(0, BufferLowAve);
    SetIndexBuffer(1, BufferHighAve);
    SetIndexBuffer(2, BufferOpenAve);
    SetIndexBuffer(3, BufferCloseAve);
    SetIndexBuffer(4, BufferLowRaw);
    SetIndexBuffer(5, BufferHighRaw);
    SetIndexBuffer(6, BufferOpenRaw);
    SetIndexBuffer(7, BufferCloseRaw);
    
    SetIndexLabel(0, "Heiken Low, ave");
    SetIndexLabel(1, "Heiken High, ave");
    SetIndexLabel(2, "Heiken Open, ave");
    SetIndexLabel(3, "Heiken Close, ave");
    SetIndexLabel(4, "Heiken Low, raw");
    SetIndexLabel(5, "Heiken High, raw");
    SetIndexLabel(6, "Heiken Open, raw");
    SetIndexLabel(7, "Heiken Close, raw");
    
    SetIndexStyle(0, DRAW_HISTOGRAM);
    SetIndexStyle(1, DRAW_HISTOGRAM);
    SetIndexStyle(2, DRAW_HISTOGRAM);
    SetIndexStyle(3, DRAW_HISTOGRAM);
    
    int n = MaPeriod;
    SetIndexDrawBegin(0, n);
    SetIndexDrawBegin(1, n);
    SetIndexDrawBegin(2, n);
    SetIndexDrawBegin(3, n);
}

//----------------------------------------------------------------------
double self(int mode, int x)
{
    double v = iCustom(NULL, TimeFrame, sIndSelf,
		       0,
		       MaMethod,
		       MaPeriod,
		       MaMethod2,
		       MaPeriod2,
		       yOffset,
		       nMaxBars, mode, x);
    
    return(v);
}

//----------------------------------------------------------------------
void start()
{
    int limit;
    int counted_bars = IndicatorCounted();
    
    if (counted_bars > 0) {
	counted_bars--;
    }
    
    limit = Bars - counted_bars;
    int limit0 = limit;
    if (nMaxBars > 0) {
	limit = MathMin(limit, nMaxBars);
    }
    
    // clear beyond limits
    for (int i = limit0 - 1; i >= limit; i--) {
	BufferLowAve[i]   = 0;
	BufferHighAve[i]  = 0;
	BufferOpenAve[i]  = 0;
	BufferCloseAve[i] = 0;
	BufferLowRaw[i]   = 0;
	BufferHighRaw[i]  = 0;
	BufferOpenRaw[i]  = 0;
	BufferCloseRaw[i] = 0;
    }
    
    if (TimeFrame != Period()) {
	// MTF
	limit = MathMax(limit, TimeFrame / Period());
	for (i = limit - 1; i >= 0; i--) {
	    int x = iBarShift(NULL, TimeFrame, Time[i]);
	    BufferLowAve[i]   = self(0, x);
	    BufferHighAve[i]  = self(1, x);
	    BufferOpenAve[i]  = self(2, x);
	    BufferCloseAve[i] = self(3, x);
	}
	
	return;
    }
    
    // TimeFrame == Period()
    for (i = limit - 1; i >= 0; i--) {
	double maOpen  = iMA(NULL, 0, MaPeriod, 0, MaMethod, MODE_OPEN,  i) + yOffset * Point;
	double maClose = iMA(NULL, 0, MaPeriod, 0, MaMethod, MODE_CLOSE, i) + yOffset * Point;
	double maHigh  = iMA(NULL, 0, MaPeriod, 0, MaMethod, MODE_HIGH,  i) + yOffset * Point;
	double maLow   = iMA(NULL, 0, MaPeriod, 0, MaMethod, MODE_LOW,   i) + yOffset * Point;
	
	double haOpen  = (BufferOpenRaw[i + 1] + BufferCloseRaw[i + 1]) * 0.5;
	double haClose = (maOpen + maHigh + maLow + maClose) * 0.25;
	double haHigh  = MathMax(maHigh, MathMax(haOpen, haClose));
	double haLow   = MathMin(maLow,  MathMin(haOpen, haClose));
	
	if (haOpen < haClose) {
	    BufferLowRaw[i]  = haLow;
	    BufferHighRaw[i] = haHigh;
	} else {
	    BufferLowRaw[i]  = haHigh;
	    BufferHighRaw[i] = haLow;
	} 
	BufferOpenRaw[i]  = haOpen;
	BufferCloseRaw[i] = haClose;
    }
    
    for (i = limit - 1; i >= 0; i--) {
	haLow   = iMAOnArray(BufferLowRaw,   0, MaPeriod2, 0, MaMethod2, i);
	haHigh  = iMAOnArray(BufferHighRaw,  0, MaPeriod2, 0, MaMethod2, i);
	haOpen  = iMAOnArray(BufferOpenRaw,  0, MaPeriod2, 0, MaMethod2, i);
	haClose = iMAOnArray(BufferCloseRaw, 0, MaPeriod2, 0, MaMethod2, i);
	
	if (haLow > haHigh) {
	    double tmp = haLow;
	    haLow = haHigh;
	    haHigh = tmp;
	}
	
	if (haOpen < haLow) {
	    haLow = haOpen;
	}
	if (haOpen > haHigh) {
	    haHigh = haOpen;
	}
	
	if (haClose < haLow) {
	    haLow = haClose;
	}
	if (haClose > haHigh) {
	    haHigh = haClose;
	}
	
	if (haOpen < haClose) {
	    if (haLow < haHigh) {
		BufferLowAve[i]   = haLow;
		BufferHighAve[i]  = haHigh;
	    } else {
		BufferLowAve[i]   = haHigh;
		BufferHighAve[i]  = haLow;
	    }
	} else {
	    if (haLow < haHigh) {
		BufferLowAve[i]   = haHigh;
		BufferHighAve[i]  = haLow;
	    } else {
		BufferLowAve[i]   = haLow;
		BufferHighAve[i]  = haHigh;
	    }
	}
	BufferOpenAve[i]  = haOpen;
	BufferCloseAve[i] = haClose;
    }
}
